/***************************************************************************************************/
/* ppbeard.inc  - Copyright Chris Bartlett 01 Aug 2004                                             */
/* This work is licensed for royalty free re-use under the terms of the POV-Person License.        */
/* To view a copy of this license, visit http://www.telinco.co.uk/c_bartlett/license/license.htm   */
/***************************************************************************************************/
#ifndef (ppModelPrefix) #declare ppModelPrefix = "ppmx"; #end

#if (file_exists(concat(ppModelPrefix,"beardline.inc"))) #include concat(ppModelPrefix,"beardline.inc")     
#else #declare ppBeardLine = sphere {<0,50,30>,40}
#end
#include "transforms.inc"

#declare ppBeardLine = union {
  object{ppBeardLine} 
  object{ppBeardLine scale <-1,1,1>}  
  translate -y*50
  scale 1.01      
  translate y*50
}


#ifndef (ppBeardInvertNormal)     #declare ppBeardInvertNormal   = 1;            #end     // 1 = outward facing ppBeardLine object, 2 = inward facing ppBeardLine object
#ifndef (ppBeardHairNumber)       #declare ppBeardHairNumber     = 1000;         #end
#ifndef (ppBeardHairThickness)    #declare ppBeardHairThickness  = 0.005;        #end
#ifndef (ppBeardHairWidth)        #declare ppBeardHairWidth      = 1;            #end
#ifndef (ppBeardHairLength)       #declare ppBeardHairLength     = 0.2;          #end          // 0 to 2
#ifndef (ppBeardMoustachLength)   #declare ppBeardMoustachLength = 0.4;            #end      // 0 to 0.8
#ifndef (ppBeardMoustachDef)      #declare ppBeardMoustachDef    = <150,20,-10>; #end
#ifndef (ppBeardMoustachSplay)    #declare ppBeardMoustachSplay  = 1;            #end
#ifndef (ppBeardExclude)          #declare ppBeardExclude        = 0;            #end           // 0=Include Every Hair, 1=Exclude Main Hair, 2=Exclude Fringe, 3=Both
#ifndef (ppBeardCentre)           #declare ppBeardCentre         = <0,50,30>;    #end    // The logical centre from which hairs radiate
#ifndef (ppBeardSeed)             #declare ppBeardSeed           = 1; #else #declare ppBeardSeed = ppBeardSeed + 1; #end
#ifndef (ppBeardTexture)      
  #ifndef (ppBeardColour)         #declare ppBeardColour = <1,1,0>; #end 
  #declare ppBeardTexture = texture {pigment {color ppBeardColour} finish {phong 0.35}};
#end 
#ifndef (ppMoustachTexture)
  #ifndef (ppMoustachColour)     #declare ppMoustachColour = ppBeardColour; #end 
  #declare ppMoustachTexture = texture {pigment {color ppMoustachColour} finish {phong 0.35}};
#end 
#ifndef (ppBeardLineVisible) #declare ppBeardLineVisible = 1; #end        // 0=Invisible, 1=Hairs Visible, 2=Solid Hair Colour

#declare ppBeard = union {

#switch (ppBeardLineVisible)
  #case (1)
    object {ppBeardLine scale <-1,1,1>
      texture { 
        pigment {spotted 
          color_map {
            [0    color <1,1,1,0.95>]
            [0.6  color <1,1,1,0.95>]
            [0.6  color ppBeardColour+<0,0,0,0.6>]
            [1    color ppBeardColour+<0,0,0,0.5>]
          } 
          scale 0.1
        }
      finish {phong 0.05}
      }  
    }
  #break
  #case (2)
    object {ppBeardLine scale <-1,1,1> 
      texture {
        pigment {color ppBeardColour+<0,0,0,0.6>}  // Add a bit of transparency
        finish {phong 0.05}
      }  
    }
  #break
#end


#local ppBeardHairNormal = <0,0,0>;
#local ppBeardSeedRef = seed(ppBeardSeed); 
#local ppBeardLineCentred = object {ppBeardLine translate -ppBeardCentre}


#if (ppBeardExclude=3) #local ppBeardHairCount = ppBeardHairNumber; #else #local ppBeardHairCount = 0; #end
#while (ppBeardHairCount < ppBeardHairNumber)   
  #local ppBeardHairDirection = <2*rand(ppBeardSeedRef)-1,2*rand(ppBeardSeedRef)-1,2*rand(ppBeardSeedRef)-1>;
  #local ppBeardHairRootPos = trace (ppBeardLineCentred, ppBeardHairDirection,0-ppBeardHairDirection,ppBeardHairNormal);
  #if (vlength(ppBeardHairNormal)!=0)          // i.e. If we hit the surface  
    #local ppBeardHairNormal = -ppBeardHairNormal;
    #local ppBeardHairOccurs   = 1;
    #local ppBeardHairInMoustach = 0;   
    #if (ppBeardHairRootPos.x >= -ppBeardMoustachDef.x 
       & ppBeardHairRootPos.x <= ppBeardMoustachDef.x 
       & ppBeardHairRootPos.y >= ppBeardMoustachDef.y 
       & ppBeardHairRootPos.z <= ppBeardMoustachDef.z) 
       #local ppBeardHairInMoustach = 1; #end

//#debug concat(str(ppBeardHairInMoustach,3,3),"\n")
    #if (ppBeardExclude = 1 & !ppBeardHairInMoustach) #local ppBeardHairOccurs = 0; #end
    #if (ppBeardExclude = 2 & ppBeardHairInMoustach)  #local ppBeardHairOccurs = 0; #end
    #if (ppBeardHairOccurs) 
//      #if (abs(ppBeardHairRootPos.y) < 30) #declare ppBeardHairThicknessScale = 3-abs(ppBeardHairRootPos.y)/15; #else 
//      #declare ppBeardHairThicknessScale = 1; //#end 
      #if (ppBeardInvertNormal) #local ppBeardHairNormal = -ppBeardHairNormal;  #end // Invert the Normal to point the hair outwards        
//      #declare ppBeardHairThickness = ppBeardHairThickness*ppBeardHairThicknessScale;
      #local ppBeardHairBoxSize = 0.4 + ppBeardHairThickness;
      intersection {
        #if (!ppBeardHairInMoustach)                                           // A hair that is part of the Main beard
          torus {0.1,ppBeardHairThickness rotate z*90 translate z*0.1 scale <ppBeardHairWidth,1,ppBeardHairLength>}
          box {<-ppBeardHairThickness*ppBeardHairWidth,0,-ppBeardHairThickness><ppBeardHairThickness*ppBeardHairWidth,ppBeardHairBoxSize,ppBeardHairLength+ppBeardHairThickness>}
          rotate y*45*(0.5-rand(ppBeardSeedRef))  
          texture {ppBeardTexture translate ppBeardHairRootPos}
        #else                                                         // A hair that is part of the moustach
          torus {0.1,ppBeardHairThickness rotate z*90 translate z*0.1 scale <ppBeardHairWidth,1,ppBeardMoustachLength*(1+abs(ppBeardHairRootPos.x)/40)> }
//#debug concat(str(ppBeardHairRootPos.x,3,3),"\n")
//#debug concat(str(1+abs(ppBeardHairRootPos.x)/40,3,3),"\n")
          box {<-ppBeardHairThickness*ppBeardHairWidth,0,-ppBeardHairThickness><ppBeardHairThickness*ppBeardHairWidth,1,ppBeardMoustachLength*(1+abs(ppBeardHairRootPos.x)/40)>} // Gets longer at the ends
          rotate  y*5*(0.5-rand(ppBeardSeedRef))
          rotate  x*30*abs(ppBeardHairRootPos.x)/40  
          rotate -z*90*ppBeardHairRootPos.x/40
          rotate -y*90*ppBeardHairRootPos.x/40
          texture {ppMoustachTexture translate ppBeardHairRootPos}                 // The moustach can have a different texture if needed
        #end
        scale 20 // * (ppBeardHairRootPos.y+ppBeardCentre.y)/100
        #if (!ppBeardHairInMoustach)                                           // A hair that is part of the main beard
          Reorient_Trans(<0,0,1>, <ppBeardHairRootPos.x,0,ppBeardHairRootPos.z>)
          #declare ppBeardHairRotationByPosition = 3*ppBeardHairRootPos.x - 10 - abs(ppBeardHairRootPos.z);
          #if (ppBeardHairRotationByPosition > 0) rotate -y*ppBeardHairRotationByPosition #end
        #else                                                         // A hair that is part of the moustach
          rotate x*(14+abs(ppBeardHairRootPos.x)*0.8-ppBeardMoustachSplay) 
          Reorient_Trans(<0,0,1>, <ppBeardHairRootPos.x,0,ppBeardHairRootPos.z>)
        #end
        Reorient_Trans(<0,1,0>, ppBeardHairNormal) 
        translate ppBeardHairRootPos+ppBeardCentre 
      } 
    #end
  #local ppBeardHairCount = ppBeardHairCount + 1;

  #end                     
#end

}

#ifndef (ppCamera)
  #declare ppCamera=2;
  camera {
    location <30, 70, -180>
    look_at <0, 50, 10>
    angle 34.3775
  }
  
  light_source { <0, 1400,-1600> color rgb<1, 1, 1>*1.2 }   
  
  plane {y,8
    pigment {color rgb <0.5,0.7,0.7>}  normal {bumps 0.4 scale 100}
    translate z*1000
  } 
  union {                    
    object {ppBeard}    
//    object {beardLine pigment {color <1,0.8,0.8>}}   
  
    sphere {ppBeardCentre ,5 texture {pigment {color <0,1,1>}}}
    rotate -y*60
  }
  
#end             


