/***************************************************************************************************/
/* pplips.inc  - Copyright Chris Bartlett 3 Mar 2005                                               */
/* This work is licensed under the Creative Commons Attribution-NonCommercial-ShareAlike License.  */
/* To view a copy of this license, visit http://creativecommons.org/licenses/by-nc-sa/1.0/ or      */
/* send a letter to Creative Commons, 559 Nathan Abbott Way, Stanford, California 94305, USA.      */
/***************************************************************************************************/ 
#ifndef(ppCamera)   #declare ppCamera="SantaLips";       #end
#ifndef(POVPerson)  #include "povperson.inc"        #end 
#ifndef(ppLimbScan) #include "ppclothingmacros.inc" #end 


#declare ppVerbose=0;
#declare ppShowPoints=0;
#ifndef (ppLipTexture) #declare ppLipTexture = texture {pigment {color rgb <1,0,0>}} #end
#declare ppRadius = 0.1; 

//  Imagine the lip sliced length ways, then each slice divided into segments along the length.
#local ppMaxLipSegments = 30;
#local ppkDontJoinEnds = 0;

#declare ppUpperLipNormal = array[ppLipSlices][ppMaxLipSegments]; 
#declare ppLowerLipNormal = array[ppLipSlices][ppMaxLipSegments]; 


#if (ppVerbose = 1) #debug concat("Start of Upper Lip","\n")  #end 
#declare ppLipProfile = array[ppLipSlices-1] {
  <0,-0.002, 0.000>,
  <0,-0.010, 0.010>,
  <0,-0.010, 0.020>,
  <0,-0.010, 0.030>,
  <0,-0.010, 0.040>,
  <0, 0.010, 0.040>,
  <0, 0.010, 0.020>,
  <0, 0.010, 0.000>
}
// Read the line of the upper lip from the first slice of a pair of arrays, then extrude along a profile.
ppExtrudeLipAlongProfile (ppUpperLipArray, ppUpperLipNormal, ppLipSlices, ppUpperLipSegments, ppLipProfile,-1)
#declare ppUpperLip = ppMesh2FromArray(ppUpperLipArray,ppUpperLipNormal,ppLipSlices,ppUpperLipSegments,ppkDontJoinEnds)


#if (ppVerbose = 1) #debug concat("Start of Lower Lip","\n")  #end 
#declare ppLipProfile = array[ppLipSlices-1] {
  <0, 0.001, 0.000>,
  <0, 0.005, 0.005>,
  <0, 0.005, 0.010>,
  <0, 0.005, 0.015>,
  <0, 0.005, 0.020>,
  <0,-0.005, 0.020>,
  <0,-0.005, 0.010>,
  <0,-0.005, 0.000>
}
//  Read the line of the lower lip from the first slice of a pair of arrays, then extrude it along a profile.
ppExtrudeLipAlongProfile (ppLowerLipArray, ppLowerLipNormal, ppLipSlices, ppLowerLipSegments, ppLipProfile, 1)
#declare ppLowerLip = ppMesh2FromArray(ppLowerLipArray,ppLowerLipNormal,ppLipSlices,ppLowerLipSegments,ppkDontJoinEnds)


#declare ppLips = union {object {ppUpperLip} object {ppLowerLip}} 
                                                 
#if (strcmp(ppCamera,"SantaLips")=0)  
  #declare ppShowPoints = 1;
  #debug concat("Lip: ",vstr(3,ppUpperLipArray[0][0],",",0,3) ,"\n")         
//  #if (ppShowPoints=1) ppDrawPoints(ppUpperLipArray,ppLipSlices,ppUpperLipSegments,0.1,<0,3,3>) #end    

  #local ppMin = min_extent (ppLips);
  #local ppMax = max_extent (ppLips);
  #local ppCenter = ppMin + (ppMax-ppMin)/2;


  object {ppUpperLip texture {ppLipTexture}}
  object {ppLowerLip texture {ppLipTexture}}

  light_source { <2, 60,-10> color rgb<1, 1, 1>*1.4 }   
  light_source { < 100, 14,-100> color rgb<1, 1, 1>*1.2 }   
  light_source { <-100, 14,10> color rgb<1, 1, 1>*1.2 }   
  camera {location ppCenter-30*x look_at ppCenter} // look_at <11,56,-16>}
#end   
