/***************************************************************************************************/
/* ppshortwavyhair.inc  - Copyright Chris Bartlett Oct 2004                                        */
/* This work is licensed for royalty free re-use under the terms of the POV-Person License.        */
/* To view a copy of this license, visit http://www.telinco.co.uk/c_bartlett/license/license.htm   */
/***************************************************************************************************/
#ifndef (ppModelPrefix) #declare ppModelPrefix = "ppf1"; #end
#ifndef (ppGender) #declare ppGender = "female"; #end

#include concat(ppModelPrefix,"hairline.inc")  
#include "transforms.inc"

#declare ppHairLine = union {object{ppHairLine} object{ppHairLine scale <-1,1,1>}}


#ifndef (ppHairNumber)       #declare ppHairNumber = 800;       #end
#ifndef (ppHairThickness)    #declare ppHairThickness = 0.005;  #end
#ifndef (ppHairLength)       #declare ppHairLength = 0.5;       #end          // 0 to 2
#ifndef (ppHairRandomness)   #declare ppHairRandomness = 5;     #end    // Degrees left or right that the hair can turn

#ifndef (ppHairFringeLength) #declare ppHairFringeLength = 2;   #end      // 0 to 0.8
#ifndef (ppHairFringeDef)    #declare ppHairFringeDef = <150,20,-10>; #end
#ifndef (ppHairFringeSplay)  #declare ppHairFringeSplay = 1;    #end
#ifndef (ppHairExclude)      #declare ppHairExclude = 0;        #end           // 0=Include Every Hair, 1=Exclude Main Hair, 2=Exclude Fringe, 3=Both
#ifndef (ppHairCentre)       #declare ppHairCentre = <0,90,30>; #end    // The logical centre from which hairs are radiate
#ifndef (ppHairSeed)         #local   ppHairSeed = 1; #else #local ppHairSeed = ppHairSeed + 1; #end
#ifndef (ppHairTexture)      
  #ifndef (ppHairColour)     #declare ppHairColour = <1,1,0>; #end 
  #declare ppHairTexture = texture {pigment {color ppHairColour} finish {phong 0.35}};
#end 
#ifndef (ppHairLineVisible) #declare ppHairLineVisible = 3; #end        // 0=Invisible, 1=Hairs Visible, 2=Solid Hair Colour

#declare ppHair = union {          // Start of ppHair declaration

  #switch (ppHairLineVisible)
    #case (1)
      object {ppHairLine scale <-1,1,1>
        texture { 
          pigment {spotted 
            color_map {
              [0    color <1,1,1,0.95>]
              [0.6  color <1,1,1,0.95>]
              [0.6  color ppHairColour+<0,0,0,0.6>]
              [1    color ppHairColour+<0,0,0,0.5>]
            }
            scale 0.1 
          }
        finish {phong 0.05}
        }  
      }
    #break
    #case (2)
      object {ppHairLine scale <-1,1,1> 
        texture {
          pigment {color ppHairColour+<0,0,0,0.6>}  // Add a bit of transparency
          finish {phong 0.05}
        }  
      }
    #break
    #case (-1)
      object {ppHairLine scale <-1,1,1> 
        texture {
          pigment { cylindrical
            color_map {
              [0    color ppHairColour+<0,0,0,0.5>]
              [0.55 color ppHairColour+<0,0,0,0.5>]
              [0.75  color <1,1,1,1>]
              [1    color <1,1,1,1>]
            }
            scale <100,1,200> rotate <0,-45,90> translate ppHairCentre
          }
          finish {phong 0.05}
        }  
        texture { 
          pigment {spotted 
            color_map {
              [0    color <1,1,1,0.95>]
              [0.6  color <1,1,1,0.95>]
              [0.6  color ppHairColour+<0,0,0,0.6>]
              [1    color ppHairColour+<0,0,0,0.5>]
            }
            scale 0.1 
          }
          finish {phong 0.05}
        }  
      }
    #break
    #else
  #end
  
  
  #local ppHairNormal = <0,0,0>;
  #local ppHairSeedRef = seed(ppHairSeed); 
  #local ppHairLineCentred = object {ppHairLine translate -ppHairCentre} 
  #local ppHairRotateByDistance = 30;
//  #if (strcmp(ppGender,"male")=0) #local ppHairRotateByDistance = 0.65;
//  #else #local ppHairRotateByDistance = 1; 
//  #end
  
  // Theoretically, clustering hairs should provide performance improvements. In reality it only seems to make a 
  // difference for large number of hairs.
  // Although the algorithm gets a little more complicated, the ppHairDirection variable used by the trace command 
  // should miss the hairline object less often. Furthermore, the automatic heirarchy of bounding boxes should reduce 
  // render times when compared with a completely random set of values.
  // I think there is potential for reducing this further by replacing the random algorithm by a scanning algorithm
  // with a randomised deviation from the scan path. The problem that would need to be resolved is how to get the  
  // right number of hairs to hit the surface.
//  #local ppHairClusterSize = ceil(sqrt(ppHairNumber));
  #local ppHairClusterSize = ceil(pow(ppHairNumber,1/3));
  #local ppHairClusterCubeRoot = pow(ppHairClusterSize,1/3);

  #if (ppHairExclude=3) #local ppHairCount = ppHairNumber; #else #local ppHairCount = 0; #end
  #while (ppHairCount < ppHairNumber)

    #if (ppHairCount/ppHairClusterSize = floor(ppHairCount/ppHairClusterSize))   
      #local ppHairDirection = <2*rand(ppHairSeedRef)-1,2*rand(ppHairSeedRef)-1,2*rand(ppHairSeedRef)-1>;
      #local ppHairClusterDirection = ppHairDirection;
      // Make this patch of hair darker or lighter by up to 10%
      #declare ppHairTexture = texture {pigment {color ppHairColour*(0.5+rand(ppHairSeedRef)/1)} finish {phong 0.35}};
    #else
      #local ppHairDirection = <ppHairClusterDirection.x+(2*rand(ppHairSeedRef)-1)/ppHairClusterCubeRoot,
                                ppHairClusterDirection.y+(2*rand(ppHairSeedRef)-1)/ppHairClusterCubeRoot, 
                                ppHairClusterDirection.z+(2*rand(ppHairSeedRef)-1)/ppHairClusterCubeRoot>;
    #end
    #local ppHairDirection = vaxis_rotate(vaxis_rotate(ppHairDirection,z,45),x,45);
    #local ppHairRootPos = trace (ppHairLineCentred, ppHairDirection,0-ppHairDirection,ppHairNormal);
    #if (vlength(ppHairNormal)!=0)          // i.e. If we hit the surface
      #local ppHairOccurs   = 1;
      #local ppHairInFringe = 0;   
      #if (ppHairRootPos.x >= -ppHairFringeDef.x 
         & ppHairRootPos.x <= ppHairFringeDef.x 
         & ppHairRootPos.y >= ppHairFringeDef.y 
         & ppHairRootPos.z <= ppHairFringeDef.z) 
         #local ppHairInFringe = 1; #end 
#local ppHairInFringe = 0;
      #if (ppHairExclude = 1 & !ppHairInFringe) #local ppHairOccurs = 0; #end
      #if (ppHairExclude = 2 & ppHairInFringe)  #local ppHairOccurs = 0; #end
      #if (ppHairOccurs) 
//        #if (abs(ppHairRootPos.y) < 30) #local hairThicknessScale = 3-abs(ppHairRootPos.y)/15; #else #local hairThicknessScale = 1; #end 
        //  If the 
        #if (vlength(ppHairRootPos+ppHairNormal) < vlength(ppHairRootPos)) 
          #local ppHairNormal = -ppHairNormal;  // Invert the Normal to point the ppHair outwards 
        #end       
        #local ppHairThick = ppHairThickness; //*hairThicknessScale;
        #local ppHairBoxSize = 0.4 + ppHairThick; 
        #local ppTempAngle = 90+atan2d(ppHairNormal.y,abs(sqrt(ppHairNormal.x*ppHairNormal.x+ppHairNormal.z*ppHairNormal.z)));
        #if (!ppHairInFringe)          // A ppHair that is part of the Main Hair 
          union {
            intersection {
              torus {0.4,ppHairThick rotate z*90} 
              plane {-y,0}
              plane {-<0,ppHairNormal.y,-abs(sqrt(ppHairNormal.x*ppHairNormal.x+ppHairNormal.z*ppHairNormal.z))>,0}
              translate z*0.4  
            }
            intersection {
              torus {0.4,ppHairThick rotate z*90} 
              plane {-y,0}
              plane {-y,0 rotate x*20} 
              translate -0.4*z
              #local ppHairRotationByPosition = ppHairRotateByDistance *(ppHairRootPos.x/30);
//              #if (ppHairRotationByPosition > 0) rotate y*ppHairRotationByPosition #end
              rotate y*ppHairRotationByPosition  
              translate 0.4*z
              translate -z*0.8 rotate x*ppTempAngle translate z*0.4  
            }
            rotate x*12
            texture {ppHairTexture} // translate ppHairRootPos}
//            scale 20 * (ppHairRootPos.y+ppHairCentre.y)/100
            scale ((150+0.8*ppHairRootPos.y+ppHairCentre.y-ppHairRootPos.z))/10
//            Reorient_Trans(<0,0,1>, <ppHairRootPos.x,0,ppHairRootPos.z>)
            Reorient_Trans(<0,0,1>, <ppHairNormal.x,0,ppHairNormal.z>)
            rotate y*(0.5-rand(ppHairSeedRef))*2*ppHairRandomness  

//            Reorient_Trans(y, <0,ppHairNormal.y,ppHairNormal.z>)
//            Reorient_Trans(z, <ppHairNormal.x,0,ppHairNormal.z>)
            Reorient_Trans(<0,1,0>, ppHairNormal) 
            translate ppHairRootPos+ppHairCentre 
          }                                   
        #else                          // A ppHair that is part of the Fringe
          intersection {
            torus {0.4,ppHairThick rotate z*90 translate z*0.4 scale <1,1,1.75>}
            box {<-ppHairThick,0,-ppHairThick><ppHairThick,1,ppHairFringeLength>}  
            texture {ppHairTexture translate ppHairRootPos}                 // The fringe can have a different texture if needed
            scale 20 * (ppHairRootPos.y+ppHairCentre.y)/100
            rotate x*(14+abs(ppHairRootPos.x)*0.8-ppHairFringeSplay) 
            Reorient_Trans(<0,0,1>, <ppHairRootPos.x,0,ppHairRootPos.z>)
            rotate y*(0.5-rand(ppHairSeedRef))*2*ppHairRandomness  
            Reorient_Trans(<0,1,0>, ppHairNormal) 
            translate ppHairRootPos+ppHairCentre 
          }                                   
        #end

      #end
    #local ppHairCount = ppHairCount + 1;
  
    #end                     
  #end 
  
}   // End of ppHair declaration

#ifndef (ppCamera)
  #declare ppCamera=2;
  camera {
    location <0, 80, -220>
    look_at <0,90,30>
    angle 34.3775
  }
  
  light_source { <0, 1400,-1600> color rgb<1, 1, 1>*1.2 }   
  
  plane {y,8
    pigment {color rgb <0.5,0.7,0.7>}  normal {bumps 0.4 scale 100}
    translate z*1000
  } 
  union {                    
    object {ppHair rotate y*90}    
//    object {ppHairLine pigment {color <1,0.8,0.8>}}   
  
    sphere {ppHairCentre ,5 texture {pigment {color <0,1,1>}}}
    rotate -y*0
  }
  
#end             


